#include <View.h>
#include <Message.h>
#include <stdio.h>
#include <Handler.h>
#include "LControl.h"
#include "LView.h"

#define DH_CK	if (fDrawHost == NULL || fDrawHost->Window() == NULL)return
#define RDH_CK(x)if (fDrawHost == NULL|| fDrawHost->Window() == NULL)return x

LControl :: LControl(BRect r, BMessage * msg)
	:BInvoker(msg, NULL), BHandler(), 
	fMouseOver(false),
	fHasFocus(false),
	fIsEnabled(true),
	fDrawHost(NULL),
	fBounds(r),// gets reset
	fFrame(r),
	fValue(0),
	fTracking(false),
	fControlFont(NULL),
	fParentControl(NULL)
{	
	ResBounds();
	fMembers = new BList(1);
	fVisible = true;
	fMinW = 0;
	fMaxW = 65536;
	fMinH = 0;
	fMaxH = 65536;
}

void	LControl	::	Hide()
{
	fVisible = false;
	
	for (int i = 0; i < CountMembers(); i++)
		MemberAt(i)->Hide();
	
	Invalidate();
}

void	LControl	::	Show()
{
	fVisible = true;
	for (int i = 0; i < CountMembers(); i++)
		MemberAt(i)->Show();

	Invalidate();
}

bool	LControl	::	IsHidden()
{
	return !fVisible;
}

LControl :: ~LControl(){if (fControlFont != NULL)delete fControlFont;}

void LControl::ResBounds(){fBounds = BRect(0,0,Frame().Width()-1, Frame().Height()-1);;}

void
	LControl::SetDrawHost(BView * dH)
	{	fDrawHost = dH;	SetTarget(fDrawHost);	}

void LControl::SetEnabled(bool x)
	{
		if (IsEnabled() == x)return;
		fIsEnabled = x;Invalidate();
	}
	

void LControl::SetFocus(bool x){
	fHasFocus = x;Invalidate();
	DH_CK;fDrawHost->MakeFocus(1);// if we are shared with real children...
				// or if we need to ensure we capture keyboard events… see?
	
}

void	LControl::SetControlFont(BFont * font)
	{
		if (fControlFont != NULL)
			delete fControlFont;
		if (font == NULL){fControlFont = NULL; return;}// no need to allocate for NULL.. duh
		fControlFont = new BFont(font);
	}

BFont * LControl::ControlFont()
	{
		if (fControlFont != NULL)return fControlFont;
		return (new BFont(be_plain_font));
	}

void	LControl::SetTracking(bool x){fTracking=x;}
bool	LControl::IsTracking(){return fTracking;}

void	LControl::SetValue(int32 x)
{	ValueChanged(x);fValue = x;	}
int32	LControl::Value(){return fValue;}

void LControl::SetMouseOver(bool x){fMouseOver = x;MouseOver(fMouseOver);}

void LControl::MouseOver(bool){}

void LControl::AttachedToWindow(){}
void LControl::DetachedFromWindow(){}
void LControl::Pulse(){}
void LControl::Draw(){}
void LControl::MouseDown(BPoint){}
void LControl::MouseUp(BPoint){}
void LControl::MouseMoved(BPoint, uint32, const BMessage*){}
void LControl::KeyDown(const char*, int32){}
void LControl::KeyUp(const char*, int32){}
void LControl::MessageReceived(BMessage*){}

bool LControl::IsEnabled(){return fIsEnabled;}
bool LControl::HasFocus(){return fHasFocus;}
BRect LControl::Bounds(){return fBounds;}
bool LControl::IsMouseOver(){return fMouseOver;}
void LControl::ValueChanged(int32){}



void	LControl::MoveTo(BPoint p)
		{
			float delta_x = 0, delta_y = 0;		
				delta_x = p.x - fFrame.left;
				delta_y = p.y - fFrame.top;

			for (int i = 0; i < CountMembers(); i++)
				MemberAt(i)->MoveBy(delta_x, delta_y);

			float H = Bounds().Height(), W = Bounds().Width();
			fFrame.left = p.x; fFrame.top = p.y;
			fFrame.right = p.x + W; fFrame.bottom = p.y + H;
		
		
			Invalidate();			
		}

void	LControl::MoveTo(float x, float y){MoveTo(BPoint(x,y));}
void	LControl::MoveBy(float x, float y){
		BRect b = Frame();
		fFrame.OffsetBy(x,y);
		DH_CK;
		b.right += x;
		b.bottom += y;
		for (int i = 0; i < CountMembers(); i++)
			MemberAt(i)->MoveBy(x,y);
		
		fDrawHost->	Invalidate(b);
	
	}
		
		
void	LControl	::	SetSizeConstraints(float minW, float maxW, float minH, float maxH)
{
	fMinW = minW;	fMaxW = maxW;	fMinH = minH;	fMaxH = maxH;
}

void
	LControl	::		GetSizeConstraints(float *minW, float *maxW, float *minH, float *maxH)
	{
		*minW = fMinW;		*maxW = fMaxW;	*minH = fMinH;	*maxH = fMaxH;
	}
		
void	LControl::ResizeTo(float x, float y)
	{	
		fFrame.right = fFrame.left + x;
		fFrame.bottom = fFrame.top + y;
		fBounds.right = x-1;	fBounds.bottom = y-1;
		
		if (fFrame.Height() < fMinH && fMinH != 0){
			fFrame.bottom = fFrame.top + fMinH;
			fBounds.bottom = fBounds.top + fMinH;
			}
		
		if (fFrame.Width() < fMinW && fMinW != 0){
			fFrame.right = fFrame.left + fMinW;
			fBounds.right = fBounds.left + fMinW;		
			}		
		
		if (fFrame.Height() > fMaxH && fMaxH != 0){
			fFrame.bottom = fFrame.top + fMaxH;
			fBounds.bottom = fBounds.top + fMaxH;		
			}

		if (fFrame.Width() > fMaxW && fMaxW != 0){
			fFrame.right = fFrame.left + fMaxW;
			fBounds.right = fBounds.left + fMaxW;		
			}		


		Invalidate();
		FrameResized(x,y);
	}
void	LControl::ResizeBy(float x, float y){
	fFrame.right += x; fFrame.bottom += y;
	fBounds.right += x; fBounds.bottom += y;
	
		if (fFrame.Height() < fMinH && fMinH != 0){
			fFrame.bottom = fFrame.top + fMinH;
			fBounds.bottom = fBounds.top + fMinH;
			}
		
		if (fFrame.Width() < fMinW && fMinW != 0){
			fFrame.right = fFrame.left + fMinW;
			fBounds.right = fBounds.left + fMinW;		
			}		
		
		if (fFrame.Height() > fMaxH && fMaxH != 0){
			fFrame.bottom = fFrame.top + fMaxH;
			fBounds.bottom = fBounds.top + fMaxH;		
			}

		if (fFrame.Width() > fMaxW && fMaxW != 0){
			fFrame.right = fFrame.left + fMaxW;
			fBounds.right = fBounds.left + fMaxW;		
			}		

	Invalidate();
	FrameResized(Bounds().Width(), Bounds().Height());
	}




BRect
	LControl	::	Frame()
	{	return fFrame;	}


// Adjusts point or rect so that the calls set in the virtual Bounds() are properly
// set in the Frame() :-)  Get it?

BPoint
	LControl	::	Adj(BPoint p)
	{	BPoint ret(p);
		ret.x += Frame().left;
		ret.y += Frame().top;
		return ret;
	}

BRect
	LControl	::	Adj(BRect r)
	{
		BRect ret(r);
		ret.OffsetBy(Frame().LeftTop());
		return ret;
	}

BPoint
	LControl	::	DeAdj(BPoint p)
	{	BPoint ret(p);
		ret.x -= Frame().left;
		ret.y -= Frame().top;
		return ret;
	}


BRect
	LControl	::	DeAdj(BRect r)
	{
		BRect ret(r);
		ret.OffsetBy(Frame().LeftTop() * -1);
		return ret;
	}


BPoint*
	LControl	::	Adj(const BPoint* ar, int32 count)
	{
		BPoint *ret = new BPoint[count];
		for (int i = 0; i < count; i++)
		{
			ret[i] = ar[i];
			ret[i].x += Frame().left;
			ret[i].y += Frame().top;
		}
		return ret;
	}

// 							MEMBER SUPPORT				MEMBER SUPPORT

void
	LControl	::	SetParentControl(LControl * c)
		{fParentControl = c;}

LControl*
	LControl	::	ParentControl()
	{return fParentControl;}

void
	LControl	::	AddMember(LControl * control)
	{
		if (control == NULL)return;
		BRect r = control->Frame();
		r.OffsetBy(Frame().LeftTop());
		control->MoveTo(r.LeftTop());
		fMembers->AddItem(control);
		((LView*)fDrawHost)->AddMember(control);
		control->SetParentControl(this);
	}

void
	LControl	::	RemoveMember(LControl * control)
	{	if (control == NULL)return;
		fMembers->RemoveItem(control);
		((LView*)fDrawHost)->RemoveMember(control);	
		Invalidate();	
	}

void
	LControl	::	RemoveMemberAt(int32 indx)
	{
		if (indx > fMembers->CountItems() -1)return;
		
		LControl * c = (LControl*)fMembers->ItemAt(indx);
		if (c != NULL)
			RemoveMember(c);
	}

LControl*
	LControl	::	MemberAt(int32 indx)
	{
		if (indx > fMembers->CountItems() -1)return NULL;
		LControl * c = (LControl*)fMembers->ItemAt(indx);
		return c;	// will be magically NULL if needed
	}

int32
	LControl	::	CountMembers()
	{
		return fMembers->CountItems();
	}





	/*	lots and lots of handing back to the host	*/

void	LControl::AddChild(BView* v){DH_CK;fDrawHost->AddChild(v);}

BView	*LControl	::	FindView(BPoint p){RDH_CK(NULL);return Window()->FindView(Adj(p));}
BView	*LControl	::	FindView(const char* p){RDH_CK(NULL);return Window()->FindView(p);}

void	LControl::DragMessage(BMessage*a, BRect b, BHandler*c){DH_CK;fDrawHost->DragMessage(a,Adj(b),c);}
void	LControl::DragMessage(BMessage*a, BBitmap*b, BPoint c, BHandler *d){DH_CK;fDrawHost->DragMessage(a,b,Adj(c),d);}
void	LControl::DragMessage(BMessage*a, BBitmap*b, drawing_mode c, BPoint d, BHandler *e){DH_CK;fDrawHost->DragMessage(a,b,c,Adj(d),e);}


rgb_color	LControl::HighColor(){RDH_CK(rgb_color()); return fDrawHost->HighColor();}
rgb_color	LControl::LowColor(){RDH_CK(rgb_color()); return fDrawHost->LowColor();}

BMessage*	LControl::CurrentMessage(){RDH_CK(NULL); return fDrawHost->Looper()->CurrentMessage();}

BPoint	LControl::ConvertToScreen(BPoint x){RDH_CK(x);return fDrawHost->ConvertToScreen(Adj(x));}
void	LControl::ConvertToScreen(BPoint* x){DH_CK; *x = Adj(*x); fDrawHost->ConvertToScreen(x);}
BRect	LControl::ConvertToScreen(BRect x){RDH_CK(x);return fDrawHost->ConvertToScreen(Adj(x));}
void	LControl::ConvertToScreen(BRect* x){DH_CK; *x = Adj(*x); fDrawHost->ConvertToScreen(x);}

BPoint	LControl::ConvertFromScreen(BPoint x){RDH_CK(x);return fDrawHost->ConvertFromScreen(Adj(x));}
void	LControl::ConvertFromScreen(BPoint* x){DH_CK; *x = Adj(*x);fDrawHost->ConvertFromScreen(x);}
BRect	LControl::ConvertFromScreen(BRect x){RDH_CK(x);return fDrawHost->ConvertFromScreen(Adj(x));}
void	LControl::ConvertFromScreen(BRect* x){DH_CK; *x = Adj(*x);fDrawHost->ConvertFromScreen(x);}

void	LControl::Flush(){DH_CK;fDrawHost->Flush();}

BWindow*LControl::Window(){RDH_CK(NULL);return fDrawHost->Window();}

void	LControl::SetPenSize(float x){DH_CK;fDrawHost->SetPenSize(x);}
float	LControl::PenSize(){RDH_CK(1.0);return fDrawHost->PenSize();}


void 	LControl::GetMouse(BPoint *p, uint32 * buts, bool b){DH_CK;fDrawHost->GetMouse(p, buts, b); *p = DeAdj(*p);}
void 	LControl::LockLooper(){DH_CK;fDrawHost->LockLooper();}
void	LControl::UnlockLooper(){DH_CK;fDrawHost->UnlockLooper();}
	
void	LControl::SetHighColor(uchar r, uchar g, uchar b, uchar a)
					{DH_CK;fDrawHost->SetHighColor(r,g,b,a);}
void	LControl::SetHighColor(rgb_color col){DH_CK;fDrawHost->SetHighColor(col);}
void	LControl::SetLowColor(uchar r, uchar g, uchar b, uchar a){DH_CK;fDrawHost->SetLowColor(r,g,b,a);}
void	LControl::SetLowColor(rgb_color col){DH_CK;fDrawHost->SetLowColor(col);}

void	LControl::SetDrawingMode(drawing_mode mode){DH_CK;fDrawHost->SetDrawingMode(mode);}
drawing_mode	LControl::DrawingMode()const{DH_CK(B_OP_COPY);return fDrawHost->DrawingMode();}
	

void	LControl::MovePenTo(BPoint p){DH_CK;fDrawHost->MovePenTo(Adj(p));}
void	LControl::MovePenTo(float x, float y){MovePenTo(BPoint(x,y));}
void	LControl::MovePenBy(float x, float y){DH_CK;fDrawHost->MovePenBy(x,y);}
BPoint	LControl::PenLocation(){RDH_CK(BPoint(0,0));return DeAdj(fDrawHost->PenLocation());}

void	LControl::StrokeLine(BPoint p, pattern pat){DH_CK;fDrawHost->StrokeLine(Adj(p), pat);}
void	LControl::StrokeLine(BPoint p, BPoint pa, pattern pat){DH_CK;fDrawHost->StrokeLine(Adj(p), Adj(pa), pat);}

void	LControl::BeginLineArray(int32 c){DH_CK;fDrawHost->BeginLineArray(c);}
void	LControl::AddLine(BPoint p, BPoint pa, rgb_color col){DH_CK;fDrawHost->AddLine(Adj(p), Adj(pa), col);}
void	LControl::EndLineArray(){DH_CK;fDrawHost->EndLineArray();}

void	LControl::StrokePolygon(const BPolygon *poly, bool closed, 
				pattern pat){DH_CK;fDrawHost->StrokePolygon(poly, closed, pat);}
void	LControl::StrokePolygon(const BPoint *p, int32 count,
			bool closed, pattern pat){DH_CK;
			fDrawHost->StrokePolygon(Adj(p, count),count,closed,pat);}
void	LControl::StrokePolygon(const BPoint *p, int32 count, BRect clip,
				bool closed = true, pattern pat){DH_CK;fDrawHost->StrokePolygon(Adj(p, count),count,clip, closed,pat);}
		
void	LControl::FillPolygon(const BPolygon *p, pattern pat){DH_CK;fDrawHost->FillPolygon(p,pat);}
void	LControl::FillPolygon(const BPoint *p, int32 count,pattern pat){DH_CK;;fDrawHost->FillPolygon(Adj(p, count),count,pat);}
void	LControl::FillPolygon(const BPoint *p, int32 count, BRect clip,pattern pat){DH_CK;fDrawHost->FillPolygon(Adj(p, count),count,clip,pat);}
						
void	LControl::StrokeTriangle(BPoint a, BPoint b, BPoint c, BRect r, 
				pattern pat){DH_CK;fDrawHost->StrokeTriangle(Adj(a),Adj(b),Adj(c),Adj(r),pat);}

void	LControl::StrokeTriangle(BPoint a, BPoint b, BPoint c,
				pattern pat){DH_CK;fDrawHost->StrokeTriangle(Adj(a),Adj(b),Adj(c),pat);}

void	LControl::FillTriangle(BPoint a, BPoint b, BPoint c, BRect r, 
				pattern pat){DH_CK;fDrawHost->FillTriangle(Adj(a),Adj(b),Adj(c),Adj(r),pat);}

void	LControl::FillTriangle(BPoint a, BPoint b, BPoint c,
				pattern pat){DH_CK;fDrawHost->FillTriangle(Adj(a),Adj(b),Adj(c),pat);}

void	LControl::StrokeRect(BRect r, pattern pat){DH_CK;fDrawHost->StrokeRect(Adj(r), pat);}
void	LControl::FillRect(BRect r, pattern pat){DH_CK;fDrawHost->FillRect(Adj(r), pat);}
void	LControl::FillRegion(BRegion* r, pattern pat){DH_CK;fDrawHost->FillRegion(r, pat);}
void	LControl::InvertRect(BRect r){DH_CK;fDrawHost->InvertRect(Adj(r));}

void	LControl::StrokeRoundRect(BRect r, float xR, float yR, 
			pattern pat){DH_CK;fDrawHost->StrokeRoundRect(Adj(r), xR, yR, pat);}
void	LControl::FillRoundRect(BRect r, float xR, float yR, 
			pattern pat){DH_CK;fDrawHost->FillRoundRect(Adj(r), xR, yR, pat);}
					
void	LControl::StrokeEllipse(BPoint p, float a, float b,
					pattern pat){DH_CK;fDrawHost->StrokeEllipse(Adj(p), a, b, pat);}

void	LControl::FillEllipse(BPoint p, float a, float b,
			pattern pat){DH_CK;fDrawHost->FillEllipse(Adj(p), a, b, pat);}

void	LControl::StrokeArc(BPoint p, float a, float b,float c,float d,
			pattern pat){DH_CK;fDrawHost->StrokeArc(Adj(p), a, b, c, d, pat);}

void	LControl::StrokeArc(BRect p, float a, float b,
			pattern pat){DH_CK;fDrawHost->StrokeArc(Adj(p), a, b, pat);}

void	LControl::FillArc(BPoint p, float a, float b, float c, float d,
			pattern pat){DH_CK;fDrawHost->FillArc(Adj(p),a,b,c,d,pat);}

void	LControl::FillArc(BRect p, float a, float b,
			pattern pat){DH_CK;fDrawHost->FillArc(Adj(p),a,b,pat);}

void	LControl::StrokeBezier(BPoint* p, pattern pat){DH_CK;fDrawHost->StrokeBezier(p, pat);}
void	LControl::FillBezier(BPoint* p, pattern pat){DH_CK;fDrawHost->FillBezier(p, pat);}

void	LControl::StrokeShape(BShape*s, pattern pat){DH_CK;fDrawHost->StrokeShape(s, pat);}
void	LControl::FillShape(BShape*s, pattern pat){DH_CK;fDrawHost->FillShape(s, pat);}

void	LControl::CopyBits(BRect from, BRect to){DH_CK;fDrawHost->CopyBits(Adj(from), Adj(to));}
		
void	LControl::DrawBitmapAsync(const BBitmap *map, BRect s, BRect d){DH_CK;fDrawHost->DrawBitmapAsync(map, s,Adj(d));}
void	LControl::DrawBitmapAsync(const BBitmap*map){DH_CK;fDrawHost->DrawBitmapAsync(map);}
void	LControl::DrawBitmapAsync(const BBitmap*map, BPoint p){DH_CK;fDrawHost->DrawBitmapAsync(map, Adj(p));}
void	LControl::DrawBitmapAsync(const BBitmap*map, BRect r){DH_CK;fDrawHost->DrawBitmapAsync(map, Adj(r));}
		
void	LControl::DrawBitmap(const BBitmap *map, BRect s, BRect d){DH_CK;fDrawHost->DrawBitmap(map, s,Adj(d));}
void	LControl::DrawBitmap(const BBitmap*map){DH_CK;fDrawHost->DrawBitmap(map);}
void	LControl::DrawBitmap(const BBitmap*map, BPoint p){DH_CK;fDrawHost->DrawBitmap(map, Adj(p));}
void	LControl::DrawBitmap(const BBitmap*map, BRect r){DH_CK;fDrawHost->DrawBitmap(map,Adj(r));}
		
		// Text&Font functions

void	LControl::DrawChar(char c){DH_CK;fDrawHost->DrawChar(c);}
void	LControl::DrawChar(char c, BPoint p){DH_CK;fDrawHost->DrawChar(c, Adj(p));}
		
void	LControl::DrawString(const char* s, escapement_delta* d){DH_CK;fDrawHost->DrawString(s, d);}
void	LControl::DrawString(const char*s, BPoint p, escapement_delta* d){DH_CK;fDrawHost->DrawString(s, Adj(p), d);}
void	LControl::DrawString(const char*s, int32 c,	escapement_delta* d){DH_CK;fDrawHost->DrawString(s, c, d);}
void	LControl::DrawString(const char*s, int32 c, BPoint p, escapement_delta* d){DH_CK;fDrawHost->DrawString(s, c, Adj(p), d);}
		
void	LControl::SetFont(const BFont* f, uint32 x){DH_CK;fDrawHost->SetFont(f, x);}
void	LControl::GetFont(BFont*f) const{DH_CK;fDrawHost->GetFont(f);}
		
void	LControl::TruncateString(BString *s, uint32 x, float a) const{DH_CK;fDrawHost->TruncateString(s,x,a);}
float	LControl::StringWidth(const char* s) const{RDH_CK(0.0);return fDrawHost->StringWidth(s);}
float	LControl::StringWidth(const char*s, int32 l) const{RDH_CK(0.0);return fDrawHost->StringWidth(s, l);}
		
void	LControl::GetStringWidths(char*array[],int32 lenAR[],
			int32 numString, float widthAR[]) const{DH_CK;fDrawHost->GetStringWidths(array, lenAR, numString, widthAR);}
				
void	LControl::SetFontSize(float x){DH_CK;fDrawHost->SetFontSize(x);}
void	LControl::ForceFontFlags(uint32 x){DH_CK;fDrawHost->ForceFontFlags(x);}
void	LControl::ForceFontAliasing(bool x){DH_CK;fDrawHost->ForceFontAliasing(x);}
void	LControl::GetFontHeight(font_height*x) const{DH_CK;fDrawHost->GetFontHeight(x);}

void	LControl::Invalidate(){DH_CK;fDrawHost->Invalidate(Frame());}
void	LControl::Invalidate(BRect r){DH_CK; r=Adj(r);if (Frame().Contains(r))fDrawHost->Invalidate(r);else Invalidate();}
										
void	LControl::DelayedInvalidate(bigtime_t t, BRect r)
{DH_CK;r=Adj(r); if (Bounds().Contains(r))fDrawHost->DelayedInvalidate(t,r);else fDrawHost->DelayedInvalidate(t, Frame());}

void	LControl::InvalidateAtTime(bigtime_t t, BRect r)
{DH_CK;if (Bounds().Contains(r))fDrawHost->InvalidateAtTime(t,r);else fDrawHost->InvalidateAtTime(t, Frame());}

bool	LControl::IsInvalidatePending() const{RDH_CK(0);return fDrawHost->IsInvalidatePending();}

void	LControl::SetViewCursor(const BCursor* c, bool x){DH_CK; fDrawHost->SetViewCursor(c, x);}

void	LControl::FrameResized(float,float){}

status_t	LControl	::	SetMouseEventMask(uint32 a, uint32 b)
{
	RDH_CK(B_ERROR);
	return fDrawHost->SetMouseEventMask(a, b);

}
