/*
	LControl
		
	You can pretend that this is a BView and nealy code blindy
	if you are familiar, or by the BeBook if not.

*/

#ifndef L_CONTROL_H_
#define L_CONTROL_H_

#include <Window.h>
#include <View.h>
#include <Font.h>
#include <Message.h>
#include <Invoker.h>
#include <Handler.h>

class BCursor;

class LControl : public BInvoker, public BHandler{

	public:
		LControl(BRect virtualFrame, BMessage * = NULL);
		virtual ~LControl();

		virtual void	AttachedToWindow();
		virtual void	DetachedFromWindow();
		
		virtual void	Pulse();
							// 		****	Draw() function is not same as BView's
							//		****	This is because we normally are really good
							//		****	at determining what we should draw, and when.
		virtual void	Draw();// draw fully.. only called if needed, BTW so don't worry
			// search for "NeedsUpdate" in this file |	
			//										 V	

		virtual void	MouseDown(BPoint);
		virtual void	MouseUp(BPoint);
		virtual	void	FrameResized(float, float);

		virtual void	MouseMoved(BPoint, uint32, const BMessage*);
		virtual void	KeyDown(const char*, int32);
		virtual void	KeyUp(const char*, int32);
		
		virtual void	MessageReceived(BMessage*);

		virtual void	SetFocus(bool);// you need focus for keyboard events
		bool			HasFocus();
	
		virtual void	SetEnabled(bool);// basic control work
		bool			IsEnabled();	// if not enabled, some notices
										// will not be sent to you, 
										// default is true of course

		virtual void	SetValue(int32);
				int32	Value();
		virtual void	ValueChanged(int32 newVal);// Called BEFORE value changes.

			// ONLY acts as placeholder for specified font.
			// Meant for dynamically changing app-side font choices.
		virtual void	SetControlFont(BFont*);// copied
				BFont	*ControlFont(); // NOTE: never returns NULL
									// if our control font is NULL, we return
									// (new BFont(be_plain_font));

		virtual void	SetTracking(bool);
				bool	IsTracking();

		// LView is not the only beast that can have LControls.. so can we!
		void	AddMember(LControl*);
		void	RemoveMember(LControl*);
		void	RemoveMemberAt(int32);
		
		LControl*	MemberAt(int32);
		int32	CountMembers();

		LControl*	ParentControl();	// NULL if not a child of an LControl

		// When we say hidden here.. we mean drawing and user-inputs disabled
		bool				IsHidden();
		virtual void			Hide();
		virtual void			Show();

		BRect			Bounds();
		BRect			Frame();

		bool			IsMouseOver();
		bool			IsWindowActive();
		
		void			MoveTo(BPoint);
		void			MoveTo(float, float);
		void			MoveBy(float, float);
		
		void			ResizeTo(float, float);
		void			ResizeBy(float, float);
		
		void			SetSizeConstraints(float minW, float maxW, float minH, float maxH);
		void			GetSizeConstraints(float *minW, float *maxW, float *minH, float *maxH);		

		BView	*FindView(const char*);
		BView	*FindView(BPoint);

		BWindow*	Window();
		BMessage*	CurrentMessage();
		BPoint	ConvertToScreen(BPoint);
		void	ConvertToScreen(BPoint*);
		BRect	ConvertToScreen(BRect);
		void	ConvertToScreen(BRect*);

		BPoint	ConvertFromScreen(BPoint);
		void	ConvertFromScreen(BPoint*);
		BRect	ConvertFromScreen(BRect);
		void	ConvertFromScreen(BRect*);

		status_t	SetMouseEventMask(uint32, uint32 = 0);
		
		void	Flush();
		
		void	DragMessage(BMessage*, BRect, BHandler* = NULL);
		void	DragMessage(BMessage*, BBitmap*, BPoint, BHandler* = NULL);
		void	DragMessage(BMessage*, BBitmap*, drawing_mode, BPoint, BHandler* = NULL);
		
				
		void			GetMouse(BPoint *, uint32*, bool = true);
		void			LockLooper();
		void			UnlockLooper();

		void	SetHighColor(uchar, uchar, uchar, uchar = 255);
		void	SetHighColor(rgb_color);
		void	SetLowColor(uchar, uchar, uchar, uchar = 255);
		void	SetLowColor(rgb_color);
		
		rgb_color	HighColor();
		rgb_color	LowColor();
		
		void	SetDrawingMode(drawing_mode);
		drawing_mode	DrawingMode() const;

		// Drawing Pen:
		void	SetPenSize(float);
		float	PenSize();
		void	MovePenTo(BPoint);
		void	MovePenTo(float, float);
		void	MovePenBy(float, float);
		BPoint	PenLocation();
		
		void	StrokeLine(BPoint, pattern = B_SOLID_HIGH);
		void	StrokeLine(BPoint, BPoint, pattern = B_SOLID_HIGH);
		
		void	BeginLineArray(int32);
		void	AddLine(BPoint, BPoint, rgb_color);
		void	EndLineArray();

		void	StrokePolygon(const BPolygon *, bool closed = true, 
						pattern = B_SOLID_HIGH);

		void	StrokePolygon(const BPoint *, int32 count,
						bool closed = true, pattern = B_SOLID_HIGH);

		void	StrokePolygon(const BPoint *, int32 count, BRect clip,
						bool closed = true, pattern = B_SOLID_HIGH);
		
		void	FillPolygon(const BPolygon *,pattern = B_SOLID_HIGH);

		void	FillPolygon(const BPoint *, int32 count,pattern = B_SOLID_HIGH);

		void	FillPolygon(const BPoint *, int32 count, BRect clip,
						pattern = B_SOLID_HIGH);
						
		void	StrokeTriangle(BPoint, BPoint, BPoint, BRect, 
						pattern = B_SOLID_HIGH);

		void	StrokeTriangle(BPoint, BPoint, BPoint,
						pattern = B_SOLID_HIGH);

		void	FillTriangle(BPoint, BPoint, BPoint, BRect, 
						pattern = B_SOLID_HIGH);

		void	FillTriangle(BPoint, BPoint, BPoint,
						pattern = B_SOLID_HIGH);
		
		void	StrokeRect(BRect, pattern = B_SOLID_HIGH);
		void	FillRect(BRect, pattern = B_SOLID_HIGH);
		void	FillRegion(BRegion*, pattern = B_SOLID_HIGH);
		void	InvertRect(BRect);
		
		void	StrokeRoundRect(BRect, float xR, float yR, 
					pattern = B_SOLID_HIGH);

		void	FillRoundRect(BRect, float xR, float yR, 
					pattern = B_SOLID_HIGH);
					
		void	StrokeEllipse(BPoint, float, float,
					pattern = B_SOLID_HIGH);

		void	FillEllipse(BPoint, float, float,
					pattern = B_SOLID_HIGH);

		void	StrokeArc(BPoint, float, float,float,float,
					pattern = B_SOLID_HIGH);

		void	StrokeArc(BRect, float, float,
					pattern = B_SOLID_HIGH);

		void	FillArc(BPoint, float, float,float,float,
					pattern = B_SOLID_HIGH);

		void	FillArc(BRect, float, float,
					pattern = B_SOLID_HIGH);

		void	StrokeBezier(BPoint*, pattern = B_SOLID_HIGH);
		void	FillBezier(BPoint*, pattern = B_SOLID_HIGH);

		void	StrokeShape(BShape*, pattern = B_SOLID_HIGH);
		void	FillShape(BShape*, pattern = B_SOLID_HIGH);

		void	CopyBits(BRect, BRect);
		
		void	DrawBitmapAsync(const BBitmap *, BRect, BRect);
		
		void	DrawBitmapAsync(const BBitmap*);
		void	DrawBitmapAsync(const BBitmap*, BPoint);
		void	DrawBitmapAsync(const BBitmap*, BRect);/*suggested*/
		
		void	DrawBitmap(const BBitmap *, BRect, BRect);
		void	DrawBitmap(const BBitmap*);
		void	DrawBitmap(const BBitmap*, BPoint);
		void	DrawBitmap(const BBitmap*, BRect);
		
		// Text&Font functions

		void	DrawChar(char);
		void	DrawChar(char, BPoint);
		
		void	DrawString(const char*, escapement_delta* = NULL);
		void	DrawString(const char*, BPoint,
						escapement_delta* = NULL);
		void	DrawString(const char*, int32,
						escapement_delta* = NULL);
						
		void	DrawString(const char*, int32, BPoint,
						escapement_delta* = NULL);
		
		void	SetFont(const BFont*, uint32 = B_FONT_ALL);
		void	GetFont(BFont*) const;// not PR3 compatible :-)
		
		void	TruncateString(BString *, uint32, float) const;
		float	StringWidth(const char*) const;
		float	StringWidth(const char*, int32) const;
		
		void	GetStringWidths(char*array[],int32 lenAR[],
					int32 numString, float widthAR[]) const;
				
		void	SetFontSize(float);/*10 == system default*/
		void	ForceFontFlags(uint32);
		void	ForceFontAliasing(bool);// just for printing
		void	GetFontHeight(font_height*) const;

		void	Invalidate();
		void	Invalidate(BRect);// lots of checks, mind you
										
		void	DelayedInvalidate(bigtime_t, BRect=BRect());
		void	InvalidateAtTime(bigtime_t, BRect=BRect());
		bool	IsInvalidatePending() const;
		
		void	SetViewCursor(const BCursor*, bool = true);
		void	MakeFocus(bool x){SetFocus(x);}
		
		bool	IsFocus(){return HasFocus();}

		virtual void	MouseOver(bool);
		


/*				private usage, you need not concern yourself			*/
/*								PRIVATE									*/
/*			If you wish to use as virtual, use call-backs PLZ !			*/

		void	SetMouseOver(bool);
		void	SetDrawHost(BView*);// you get AttachedToWindow()!!

		void	AddChild(BView*);
		
		void	SetParentControl(LControl*);
		

			// if this returns false, we do not call Draw() on you!
			// the update rect is of areas that need to be 
			// redrawn, return false if you have nothing
			// in that area to be redrawn
		virtual bool	NeedsUpdate(BRect /*update*/){return true;}


	private:// for a reason, mind you
		// all of the following functions provide dynamic coordinate
		// offsets, providing for InLine processing to the BView function
		// translations.  Meaning LControl acts more like a BView than
		// any non-BView derived class, providing a familiar methodology
		// for working with the class.
		void	ResBounds();
		BPoint	Adj(BPoint);
		BRect	Adj(BRect);
		BPoint	DeAdj(BPoint);
		BRect	DeAdj(BRect);	
		BPoint*	Adj(const BPoint*, int32);

		
		float	fMinW, fMaxW, fMinH, fMaxH;
		

		bool	fVisible;
		bool	fMouseOver;
		bool	fHasFocus;
		bool	fIsEnabled;
		BView	*fDrawHost;
		BRect	fBounds;
		BRect	fFrame;
		int32	fValue;
		bool	fTracking;
		
		BFont	*fControlFont;

		// Embedded Member support
		friend class BList;
		
		BList	*fMembers;
		LControl	*fParentControl;
};

#endif

