#include <View.h>
#include <List.h>
#include <stdio.h>
#include <Bitmap.h>
#include "LControl.h"
#include "LView.h"

#define LCKCHK()	if (fLockedList && fLockFunc)return;
#define RLCKCHK(x)	if (fLockedList && fLockFunc)return x;

#define CHKLST()	if (fMembers->CountItems() <=0)return;
#define RCHKLST(x)	if (fMembers->CountItems() <=0)return x;

#define CHECK()	LCKCHK();CHKLST();
#define RCHECK(x)RLCKCHK(x);RCHKLST(x);

#define NULL_CHK(x) if (x == NULL)return;
#define RNULL_CHK(x, y) if (x == NULL)return y;

#define SPIN_LOCK()	while (fLockedList && fLockFunc)snooze(1000);


#define ITEM(x)((LControl*)fMembers->ItemAt(x))

#define TELL_EACH(x) for (int i=0;i<CountMembers();i++){CHECK();ITEM(i)->x;}
#define SPIN_TELL_EACH(x) for (int i=0;i<CountMembers();i++){SPIN_LOCK();ITEM(i)->x;}

#define RTELL_EACH(x, y) for (int i=0;i<CountMembers();i++){RCHECK(y);ITEM(i)->x;}
#define RSPIN_TELL_EACH(x, y) for (int i=0;i<CountMembers();i++){RSPIN_LOCK(y);ITEM(i)->x;}

// Nifty, eh?
#define TELL_EACH_COND(condition, x)for (int i=0;i<CountMembers();i++){CHECK();if (condition){ITEM(i)->x;}}
#define RTELL_EACH_COND(condition, x, y)for (int i=0;i<CountMembers();i++){RCHECK(y);if (condition){ITEM(i)->x;}}
#define RSPIN_TELL_EACH_COND(condition, x, y) for (int i=0;i<CountMembers();i++){RSPIN_LOCK(y);if (condition){ITEM(i)->x;}}



LView :: LView(BRect r, const char * n, uint32 rM, uint32 fl, int32 inc)
	:BView (r, n, rM, fl)
	{	fMembers = new BList(inc);
		fLockFunc = true;
		fLockedList = false;
		fHotSpot = BPoint(1,1);
		fCursor = NULL;
		SetDoubleBuffering(B_UPDATE_INVALIDATED
							|B_UPDATE_SCROLLED
							|B_UPDATE_EXPOSED
							|B_UPDATE_RESIZED);
	}

LView	::	~LView()
	{	SPIN_LOCK();// simply allow lock to be released
		fMembers->MakeEmpty();
		delete fMembers;
	}

void LView	::	DetachedFromWindow()
	{	SPIN_LOCK();
		SPIN_TELL_EACH(DetachedFromWindow());		
	}

void
	LView	::	Pulse()
	{	CHECK();
		TELL_EACH(Pulse());		
	}

void
	LView	::	Draw(BRect r)
	{	CHECK();
		//MakeFocus(true);
		TELL_EACH_COND((r.Intersects(ITEM(i)->Frame()) && ITEM(i)->NeedsUpdate(r)) && !ITEM(i)->IsHidden(), Draw())
		SetDrawingMode(B_OP_ALPHA);
	}

void
	LView	::	MouseMoved(BPoint p, uint32 t, const BMessage *m)
	{	CHECK();
		// make a new uint32 to handle the truth of inside/outside the controls
		
		for (int i = 0; i < CountMembers(); i++)
			{
				if (MemberAt(i) == NULL)break;
				if (MemberAt(i)->IsHidden() == false){
				if (MemberAt(i)->Frame().Contains(p))
					if (MemberAt(i)->IsMouseOver())
						MemberAt(i)->MouseMoved(p, B_INSIDE_VIEW, m);
					else
						MemberAt(i)->MouseMoved(p, B_ENTERED_VIEW, m);
				else if (MemberAt(i)->IsMouseOver())
					MemberAt(i)->MouseMoved(p, B_EXITED_VIEW,m);
				}
			}
				

		uint32 buts; BPoint loc;// the following takes care of the IsMouseOver stuff, obviously
		GetMouse(&loc, &buts);
		if (buts != B_MOUSE_BUTTON_1
			&& buts != B_MOUSE_BUTTON_2
			&& buts != B_MOUSE_BUTTON_3){
			TELL_EACH_COND((ITEM(i)->Frame().Contains(p)) && !ITEM(i)->IsMouseOver() && !ITEM(i)->IsHidden(), SetMouseOver(true));
			}
		TELL_EACH_COND(((ITEM(i)->IsMouseOver())&& !(ITEM(i)->Frame().Contains(p))) && !ITEM(i)->IsHidden(), SetMouseOver(false));
		
	}

void
	LView	::	MouseDown(BPoint p)
	{	CHECK();
		MakeFocus();
		TELL_EACH_COND((ITEM(i)->Frame().Contains(p))&& !ITEM(i)->IsHidden(), MouseDown(p));
	}

void
	LView	::	MouseUp(BPoint p)
	{	CHECK();
		TELL_EACH_COND((ITEM(i)->Frame().Contains(p))&& !ITEM(i)->IsHidden(), MouseUp(p));
	}

void
	LView	::	KeyDown(const char * bytes, int32 numBytes)
	{	CHECK();// handle our own key first.. we only care about tabbing

		if (!strcmp ("	", bytes) && numBytes == 1)
		{	int32 focusItem = -1;
			int32 setFocusTo = focusItem;			
			for (int i = 0; i < CountMembers(); i++)// CountMembers() does a CHECK(), so we cancel if locked in midterm
			{if (MemberAt(i)->HasFocus()){focusItem = i;MemberAt(i)->SetFocus(0);break;}}

			// find what to focus in on
			if (focusItem == -1
				&& CountMembers() != 0)
					setFocusTo = 0;
			else if (focusItem < CountMembers() -1)
					setFocusTo = focusItem + 1;
			else if (focusItem >= CountMembers() -1)
					setFocusTo = 0;// loop around
			// this work is just to ensure we don't focus a disabled item
			if (setFocusTo != -1 && MemberAt(setFocusTo)->IsEnabled())
				MemberAt(setFocusTo)->SetFocus(true);
				else
				{	int32 closest = -1;
					if (setFocusTo == -1){return;}// hmm.. would be very odd
					
					for (int i = setFocusTo; i < CountMembers(); i++)
					{	if (MemberAt(i)->IsEnabled()){
							closest = i;break;}
					}
					
					if (closest == -1)
					{	// find first enabled
						for (int i = 0; i < CountMembers(); i++)
						{	if (MemberAt(i)->IsEnabled()){
								closest = i;break;}		
						}
						if (closest == -1)return;	
					}
					MemberAt(closest)->SetFocus(true);
					
					
				}
			
				return; // do not pass on tab events
		}
		
		TELL_EACH_COND((ITEM(i)->HasFocus())&& !ITEM(i)->IsHidden(), KeyDown(bytes, numBytes));
	}

void
	LView	::	KeyUp(const char * bytes, int32 numBytes)
	{	CHECK();
		TELL_EACH_COND((ITEM(i)->HasFocus())&& !ITEM(i)->IsHidden(), KeyUp(bytes, numBytes))
	}
	
void
	LView	::	MessageReceived(BMessage * msg)
	{	CHECK();
		NULL_CHK(msg);
		if (msg->WasDropped()){// only drop on apparent intended target
			BPoint dropPoint;
			status_t err = msg->FindPoint("_drop_point_", &dropPoint);
			if (err == B_OK){// can have overlapping controls that care about this drop
				ConvertFromScreen(&dropPoint);
				TELL_EACH_COND((MemberAt(i)->Frame().Contains(dropPoint))/*if evals to true*/, 
					/*do*/MessageReceived(msg));


				return;}
		}
		TELL_EACH(MessageReceived(msg));
	}

void LView	::	Lock(){fLockedList = true;}
bool LView	::	IsLocked(){return fLockedList;}
void LView	::	Unlock(){fLockedList = false;}
void LView	::	SetFuncLockout(bool x){fLockFunc = x;}

void
	LView	::	AddMember(LControl* c)
	{	SPIN_LOCK();
		fMembers->AddItem(c);	
		c->SetDrawHost(this);
		c->AttachedToWindow();// right off the bat :-)
		c->Draw();
	}
	
int32
	LView	::	CountMembers()
	{	RCHECK(0); return fMembers->CountItems(); }

bool
	LView	::	RemoveMember(LControl * c)
	{	RCHECK(false);	
		BRect r;
		if (c == NULL)
			return false;
			
		r = c->Frame();
		fMembers->RemoveItem(c);
		Invalidate(r);// clear away the last drawn instructions
		return true;
	}

bool
	LView	::	RemoveMemberAt(int32 indx)
	{	RCHECK(false);
		LControl * c = MemberAt(indx);
		BRect r;
		if (c != NULL)
			r = c->Frame();
			else return false;
		fMembers->RemoveItem(indx);
		Invalidate(r);
		return true;
	}

LControl*
	LView	::	MemberAt(int32 indx)
	{	RCHECK(NULL);// would do a dynamic_cast, but you can't unless we recast ItemAt().. hehe useless
		return (LControl*)fMembers->ItemAt(indx);// does our checks for us
	}
	
BList*
	LView	::	Members()
	{	RCHECK(NULL);
		return fMembers;
	}

